package com.ssv.test.airbase.contest;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import com.ssv.test.airbase.contest.expressions.AssignableVariable;
import com.ssv.test.airbase.contest.expressions.ExpressionConstant;
import com.ssv.test.airbase.contest.expressions.ExpressionFunction;
import com.ssv.test.airbase.contest.expressions.ExpressionIndexed;
import com.ssv.test.airbase.contest.expressions.ExpressionOperation;
import com.ssv.test.airbase.contest.operators.ExpressionOperator;
import com.ssv.test.airbase.contest.operators.BlockOperator;
import com.ssv.test.airbase.contest.operators.GotoOperator;
import com.ssv.test.airbase.contest.operators.IfOperator;
import com.ssv.test.airbase.contest.operators.Marker;
import com.ssv.test.airbase.contest.operators.NoOperation;
import com.ssv.test.airbase.contest.operators.WhileOperator;
import com.ssv.utils.parser.ExpressionLeafProvider;
import com.ssv.utils.parser.ExpressionParser;
import com.ssv.utils.parser.Parser;
import com.ssv.utils.parser.ParserException;

/**
 * ,     CT<p>
 * 
 * <H1></H1>
 * 
 *    ,   . 
 *   : {@link com.ssv.test.airbase.contest.types}<p>
 * 
 *  ::= <br>
 *  ::= _ |
 * 				   _ |
 * 				   _if |
 * 				   _while |
 * 				   _for |
 * 				   _ |
 * 				   _goto |
 * 				   <br>
 * _ ::=  '='  <br>
 * _ ::=  '('  _ ')'<br>
 * _if ::= 'if' '('  ')'  [ 'else'  ]<br>
 * _while ::= 'while' '('  ')' <br>
 * _for ::= 'for' '(' [] ';' [] ';' [] ')' <br>
 * _ ::= '{' [  ( ';'  )] '}'<br>
 * _goto ::= 'goto' <br>
 *  ::= ':'  ':'<br>
 * 
 * @author Sergey Siryk
 */

public class ContestParser extends Parser {

	public ContestParser() throws IOException, ParserException {
		super(false);
		addSpace(' ');
		addSpace('\t');
		addSpace('\r');
		addSpace('\n');
		addCommentSymbols("/*", "*/");
		addLineCommentSymbol("//");
		addToken(LIBR, "[");
		addToken(RIBR, "]");
		addToken(LBR, "(");
		addToken(RBR, ")");
		addToken(LBLOCK, "{");
		addToken(RBLOCK, "}");
		addToken(PLUS, "+");
		addToken(MINUS, "-");
		addToken(DIV, "/");
		addToken(MUL, "*");
		addToken(POWER, "^");
		addToken(EQ, "==");
		addToken(NEQ, "!=");
		addToken(GT, ">");
		addToken(LT, "<");
		addToken(GE, ">=");
		addToken(LE, "<=");
		addToken(ASSIGN, "=");
		addToken(SEMICOLON, ";");
		addToken(COLON, ":");
		
		addToken(LAND,"&&");	
		addToken(LOR,"||");	
		addToken(LNOT,"!");	

		addKeyword(IF,"if");
		addKeyword(WHILE,"while");
		addKeyword(GOTO,"goto");
		addKeyword(ELSE,"else");
		addToken(COMMA,",");
		addKeyword(FOR,"for");
	}

	/**
	 *  , (
	 */
	public final static int LBR = 1;
	/**
	 *  , )
	 */
	public final static int RBR = 2;
	/**
	 * , +
	 */
	public final static int PLUS = 3;
	/**
	 * , -
	 */
	public final static int MINUS = 4;
	/**
	 */
	public final static int DIV = 5;
	/**
	 *  , *
	 */
	public final static int MUL = 6;
	/**
	 *  , ==
	 */
	public final static int EQ = 7;
	/**
	 *  , >
	 */
	public final static int GT = 8;
	/**
	 *  , <
	 */
	public final static int LT = 10;
	/**
	 *  , =
	 */
	public final static int ASSIGN = 12;
	/**
	 *    , ^
	 */
	public final static int POWER = 13;	
	/**
	 *   , '{'
	 */
	public final static int LBLOCK = 14;
	/**
	 *   , '}'
	 */
	public final static int RBLOCK = 15;
	/**
	 *   , ';'
	 */
	public final static int SEMICOLON = 16;
	/**
	 * , ':'
	 */
	public final static int COLON = 17;
	/**
	 *   if
	 */
	public final static int IF = 21;
	/**
	 *   while
	 */
	public final static int WHILE = 22;
	/**
	 *   goto
	 */
	public final static int GOTO = 23;
	/**
	 *  
	 */
	public final static int COMMA = 24;
	/**
	 *   , '['
	 */
	public final static int LIBR = 25;
	/**
	 *   , ']'
	 */
	public final static int RIBR = 26;

	/**
	 *  " ", &&
	 */
	public final static int LAND = 27;	

	/**
	 *  " ", ||
	 */
	public final static int LOR = 28;	

	/**
	 *  " ", !
	 */
	public final static int LNOT = 30;	
	/**
	 *   else
	 */
	public final static int ELSE = 35;			
	/**
	 *   for
	 */
	public final static int FOR = 38;				
	/**
	 *  " ", !=
	 */
	public final static int NEQ = 39;
	/**
	 *  "  ", >=
	 */
	public final static int GE = 40;
	/**
	 *  "  ", <=
	 */
	public final static int LE = 41;

	
	static class MyLeafProvider implements ExpressionLeafProvider<Expression> {
		public MyLeafProvider(ContestParser parser) {
			this.parser=parser;
		}
		
		final ContestParser parser;
		
		public Expression makeNode(Token t, Expression left,
				Expression right) {
			return new ExpressionOperation(t,left,right,parser.fetchContext());
		}
	}	
	
	final ContestExprParser expessionParser = new ContestExprParser(this); 

	public Expression parseExpression() throws ParserException, IOException {
		return expessionParser.parse(this);
	}

	public Expression acceptLValue(boolean acceptFunctionInvocation) throws IOException, ParserException {
		ParserContext ctx = this.fetchContext();
		String ident;
		List<ExpressionIndexed.Part> indexes = new ArrayList<ExpressionIndexed.Part>();
		boolean start=true;
		while( expessionParser.isLeaf(token()) ) {
			switch(token()) {
				case LIBR:
					next();
					indexes.add(new ExpressionIndexed.ExpressionPart(
							expessionParser.parse(this)));
					accept(RIBR);
					break;
				case DOUBLENUMBER:
					indexes.add(new ExpressionIndexed.NumPart(getDoubleNumber()));
					next();
					break;
				case NUMBER:
					indexes.add(new ExpressionIndexed.NumPart(getNumber()));
					next();
					break;
				case NONNUMBER:
					indexes.add(new ExpressionIndexed.StringPart(getString()));
					next();
					break;
				case IDENTIFIER:
					ident=getIdentifier();
					indexes.add(new ExpressionIndexed.StringPart(ident));
					ParserContext ictx = fetchContext();
					next();					
					if(start) {
						if(token()==LBR) { // Function invocation.
							List<Expression> l = parseParams();
							Expression expr = new ExpressionFunction(ident,l,ictx);
							return expr;
						}
					}
					break;
				default:
					new ParserException("Invalid symbol:"+stoken(),fetchContext());
			}
			start=false;
		}
		return AssignableVariable.makeVariable(indexes,ctx);
	}
	
	static class ContestExprParser extends ExpressionParser<Expression> {		
		public ContestExprParser(ContestParser parser) {
			super(new MyLeafProvider(parser));
			this.parser=parser;
			addOperatorBrackets(LBR,RBR);
			addBinary(ASSIGN);
			addBinary(LOR);
			addBinary(LAND);
			addBinary(EQ,NEQ,GT,LT,GE,LE);
			addBinary(PLUS,MINUS);
			addBinary(DIV,MUL);
			addBinaryRightToLeft(POWER);
			addUnary(LNOT);
			addUnary(MINUS);
		}
		
		/**
		 *  ,   5+5+5 - 15.
		 * @throws ParserException 
		 * @throws ParserException 
		 */
		public Expression simplify(Expression e) throws ParserException {
			if(e==null)
				return e;
			Expression el = simplify(e.getLeft());
			Expression er = simplify(e.getRight());
			if( (el!=null) && (er!=null) &&
				(el.getClass()==ExpressionConstant.class) &&
				(er.getClass()==ExpressionConstant.class)) {
				return new ExpressionConstant(e.evaluate(),e.ctx()); 
			}
			e.setLeft(el);
			e.setRight(er);
			return e;
		}		
		
		final ContestParser parser;
		
		public Expression parseLeaf(Parser parser) throws ParserException, IOException {
			return ((ContestParser)parser).acceptLValue(true);
		}

		public boolean isLeaf(int token) throws ParserException, IOException {
			return token==Parser.IDENTIFIER || 
				   token==Parser.DOUBLENUMBER ||
				   token==Parser.NUMBER ||
				   token==Parser.NONNUMBER ||
				   token==ContestParser.LIBR;
		}
	}	

	private static Logger log = Logger.getLogger("Contest");
	
	Operator compileBlock() throws ParserException, IOException {
		 accept(LBLOCK);
		 List<Operator> operators = new ArrayList<Operator>();
		 Marker lastMarker=null;
         while(token()!=RBLOCK) {
        	while(token()==SEMICOLON)
        		next();
        	if(token()!=RBLOCK) {
        		Operator operator=compile();
        		if(operator instanceof Marker)
        			lastMarker = (Marker)operator;
            	operators.add(operator);
            	if(token()!=RBLOCK)
            		accept(SEMICOLON);
        	}
         }
         next();
         if(lastMarker!=null) {
        	 Operator op = new NoOperation();
        	 operators.add(op);
         }
         return new BlockOperator(operators);
	}	
	
	Operator compile() throws ParserException, IOException {
		//accept(PRINT,PLOT,UNPLOT,IF,WHILE,IDENTIFIER,GOTO);
		switch(token()) {
			case LBLOCK: 
						log.info("BLOCK");
						return compileBlock();

			case FOR:	next();
						log.info("FOR");
						accept(LBR);
						Operator op1=null;
						Operator op2=null;
						Expression condition=null;
						if(token()!=SEMICOLON)
							op1=compile();
						accept(SEMICOLON);
						if(token()!=SEMICOLON)
							condition=parseExpression();
						accept(SEMICOLON);
						if(token()!=RBR)
							op2=compile();
						accept(RBR);
						Operator body=compile();
						
						if(condition==null)
							condition=new ExpressionConstant(DoubleOperations.TRUE,fetchContext());
						
						List<Operator> l1 = new ArrayList<Operator>();
						if(op1!=null)
							l1.add(op1);
						List<Operator> l2 = new ArrayList<Operator>();
						l2.add(body);
						if(op2!=null)
							l2.add(op2);
						l1.add(new WhileOperator(condition,new BlockOperator(l2)));
						return new BlockOperator(l1);
						
			case IF:	next();
						log.info("IF");
						accept(LBR);
						condition = parseExpression();
						accept(RBR);
						Operator operator = compile();
						if(token()==ELSE) {
							next();
							return new IfOperator(condition, operator, compile());							
						}
						return new IfOperator(condition, operator);							
						
			case WHILE: next();
						log.info("WHILE");
						accept(LBR);
						condition = parseExpression();
						accept(RBR);
						operator = compile();
						return new WhileOperator(condition, operator);
						
			case IDENTIFIER: // Expression.
			case NUMBER: // Expression.
			case DOUBLENUMBER: // Expression.
			case LIBR: // Expression.
			case NONNUMBER: // Expression.
						return new ExpressionOperator(parseExpression());
						
			case GOTO:  next();
						log.info("GOTO");						
						String gotoName = getIdentifier();
						accept(IDENTIFIER);
						return new GotoOperator(gotoName);
						
			case COLON: next();
						log.info("MARKER");						
						gotoName = getIdentifier();
						accept(IDENTIFIER);
						accept(COLON);
						return new Marker(gotoName);
			default:
						throw new ParserException("Invalid symbol:"+stoken(),
								fetchContext());
		}
	}

	ContestExprParser exprParser = new ContestExprParser(this);	
	
	/**
	 * @param numParameters -1 means no control.
	 * @return
	 * @throws IOException 
	 * @throws ParserException 
	 */
	List<Expression> parseParams() throws ParserException, IOException {
		List<Expression> parms = new ArrayList<Expression>();
		accept(LBR);
		while(token()!=RBR) {
			parms.add(exprParser.parse(this));
			if(token()!=RBR)
				accept(COMMA);
		}
		next();
		return parms;
	}

	public static void main(String[] args) {
//		log.setLevel(Level.WARNING);
		try {
			ContestParser tokens = new ContestParser();
			tokens.init(System.in);
			Operator op = tokens.compile();
			tokens.accept(EOF);
			Execution exec = new Execution(op);
			exec.run();
			System.in.read();
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println(e.getMessage());
			System.out.println("Press any key to continue.");
			try {
				System.in.read();
			} catch (IOException e1) {
			}
		}
	}

}
