package com.ssv.test.airbase.contest;

import java.util.HashMap;
import java.util.Map;

import com.ssv.test.airbase.contest.expressions.AssignableVariable;
import com.ssv.utils.parser.ParserException;

/**
 *         CT.
 * 
 * @author sergey
 *
 */
final public class DoubleOperations {
	
	static final public double TRUE = 1;
	static final public double FALSE = 0;
	
	static public Action getAction(int operation) {
		return _actions.get(operation);
	}
	
	static public interface Action {
		public double action(Expression op1, Expression op2) throws ParserException;
	}	

	static public abstract class AriphmeticAction implements Action {
		public double action(Expression op1, Expression op2) throws ParserException {
			return action(op1.evaluate(),op2.evaluate());
		}
		abstract public double action(double op1, double op2);
	}		
	
	private static final Map<Integer, Action> _actions = new HashMap<Integer, Action>();  

	static public double check(boolean condition) {
		return (condition)?TRUE:FALSE;
	}

	static public boolean check(double condition) {		
		return Double.isNaN(condition) || (condition==0);
	}	

	static private boolean nan(double op1, double op2) {
		if(Double.isNaN(op1))
			return true;
		if(Double.isNaN(op2))
			return true;
		return false;
	}
	
	static {
		_actions.put(ContestParser.ASSIGN, new Action() {
			public double action(Expression op1, Expression op2) throws ParserException {
				if(! (op1 instanceof AssignableVariable)) {
					throw new ParserException("It is not legal lvalue:"+op1,op1.ctx());
				} else {
					double d = op2.evaluate();
					((AssignableVariable)op1).getVariable().setValue(d);
					return d;
				}
			}
		});
		_actions.put(ContestParser.PLUS, new AriphmeticAction() {
			public double action(double op1, double op2) {
				return op1+op2;
			}
		});
		_actions.put(ContestParser.MINUS, new AriphmeticAction() {
			public double action(double op1, double op2) {
				return op1-op2;
			}
		});
		_actions.put(ContestParser.MUL, new AriphmeticAction() {
			public double action(double op1, double op2) {
				return op1*op2;
			}
		});
		_actions.put(ContestParser.DIV, new AriphmeticAction() {
			public double action(double op1, double op2) {
				return op1/op2;
			}
		});
		_actions.put(ContestParser.GT, new AriphmeticAction() {
			public double action(double op1, double op2) {
				if(nan(op1,op2))
					return Double.NaN;
				return check(op1>op2);
			}
		});
		_actions.put(ContestParser.LT, new AriphmeticAction() {
			public double action(double op1, double op2) {
				if(nan(op1,op2))
					return Double.NaN;
				return check(op1<op2);
			}
		});
		_actions.put(ContestParser.EQ, new AriphmeticAction() {
			public double action(double op1, double op2) {
				if(Double.isNaN(op1) && Double.isNaN(op2))
					return TRUE;
				if(nan(op1,op2))
					return Double.NaN;
				return check(op1==op2);
			}
		});
		_actions.put(ContestParser.GE, new AriphmeticAction() {
			public double action(double op1, double op2) {
				if(nan(op1,op2))
					return Double.NaN;
				return check(op1>=op2);
			}
		});
		_actions.put(ContestParser.LE, new AriphmeticAction() {
			public double action(double op1, double op2) {
				if(nan(op1,op2))
					return Double.NaN;
				return check(op1<=op2);
			}
		});
		_actions.put(ContestParser.NEQ, new AriphmeticAction() {
			public double action(double op1, double op2) {
				if(nan(op1,op2))
					return Double.NaN;
				return check(op1!=op2);
			}
		});
		_actions.put(ContestParser.POWER, new AriphmeticAction() {
			public double action(double op1, double op2) {
				if(nan(op1,op2))
					return Double.NaN;
				return Math.pow(op1,op2);
			}
		});
	}
	
	private DoubleOperations() {
	}	
}
