package com.ssv.test.airbase.contest;

import java.util.ArrayList;
import java.util.List;

import com.ssv.utils.parser.ParserException;

/**
 *    " " "" 
 * .<p>
 * 
 *      "",    
 *  {@link com.ssv.test.airbase.contest.Operator#run()} ,
 *  " " .   ,    
 * ,      goto,   
 * . ,       
 * ,      .<p>
 *     :      Step,
 *  . ""     
 *     (  Step).    
 * ,   .   -   (Goto)
 *    (ConditionalGoto).     
 *    ,       .<p>
 * 
 * @author Sergey Siryk
 *
 */

public class Execution {

	int PC = 0;
	
	private abstract static class Step {
		abstract Operator operator();
		/**
		 * @return -1 if we just go to next instruction.
		 */
		abstract int marker();
	};

	private static class SimpleStep extends Step {
		public SimpleStep(Operator op) {
			this.operator=op;
		}
		
		final Operator operator;
		
		Operator operator() {
			return operator;
		}

		public String toString() {
			return "operator:"+operator.getClass().getName();
		}
		
		int marker() {
			return -1;
		}		
	}

	public static class Goto extends Step {
		public Goto(String name) {
			markerName=name;
		}

		final String markerName;
		Integer marker=null;

		public String toString() {
			return "goto narkerName:"+(marker+1);
		}
		
		public void setMarker(int marker) {
			this.marker=marker;
		}
		
		Operator operator() {
			return null;
		}

		int marker() {
			if(marker==null)
				throw new Error("Marker "+markerName+" is not defined.");
			return marker;
		}		
	}	

	public static class ConditionalGoto extends Goto {
		public ConditionalGoto(Expression expression) {
			super(expression.toString());
			this.expression=expression;
		}

		final Expression expression;
		
		public String toString() {
			return "ifgoto "+(marker+1);
		}
		
		int marker() {
			try {
				double res = expression.evaluate();
				if(res==DoubleOperations.FALSE)
					return marker;
			} catch (ParserException e) {
				e.printStackTrace();
			}
			return -1;
		}		
	}				

	public void list() {
		int i=1;
		for(Step step:steps) {
			System.out.println(""+(i++)+" "+step.toString());
		}
	}

	public int size() {
		return steps.size();
	}

	public void add(Step step) {
		steps.add(step);
	}

	public void add(Operator op) {
		steps.add(new SimpleStep(op));
	}	
	
	List<Step> steps = new ArrayList<Step>();

	/**
	 *   Execution, "" ,  
	 *     .
	 * @param op
	 */
	public Execution(Operator op) {
		op.generate(this);
	}
	
	/**
	 *     .   
	 *   0       ,  ""   - 
	 *    . 
	 * @throws ParserException 
	 */
	public void run() throws ParserException {
		int PC=0;
		while(PC<steps.size()) {
			Step step = steps.get(PC);
			if(step.operator()!=null)
				step.operator().run();
			int next = step.marker();
			if(next==-1)
				PC++;
			else
				PC=next;
		}
	}
}
