package com.ssv.test.airbase.contest;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.ssv.test.airbase.contest.ui.ContestWindow;

/**
 * 
 *    user-defined    
 *  .<p>
 * 
 * <H1> </H1>
 * 
 * <ul>
 * <li><h3>abs(x)</h3>
 *    x.
 * </li>
 * <li><h3>cos(x)</h3>
 *   x.
 * </li>
 * <li><h3>delay(msecs)</h3>
 *      .
 * </li>
 * <li><h3>e()</h3>
 *    e (  )
 * </li>
 * <li><h3>hide()</h3>
 *    .
 * </li>
 * <li><h3>isNaN(x)()</h3>
 *    x - NaN.
 * </li>
 * <li><h3>NaN()</h3>
 *   NaN.
 * </li>
 * <li><h3>keep()</h3>
 *    .     ,  
 *      .
 * </li>
 * <li><h3>pi()</h3>
 *    PI.
 * </li>
 * <li><h3>print(x1,x2,...)</h3>
 *     ,    .
 * </li>
 * <li><h3>println(x1,x2,...)</h3>
 *     ,   
 *    .
 * </li>
 * <li><h3>plot(x,y,c,i)</h3>
 *     [x,y]   , i -  .
 *    rgb,     0  10. ,
 * 101010    (  , -).
 * </li>
 * <li><h3>random(x)</h3>
 *     0  x.  x ,  
 *   0  1.
 * </li>
 * <li><h3>readnum()</h3>
 *          ( NaN 
 *    ).
 * </li>
 * <li><h3>repaint()</h3>
 *      .   
 *           .    
 *  ,       . 
 * </li>
 * <li><h3>sin()</h3>
 *   x.
 * </li>
 * <li><h3>sqrt(x)</h3>
 *    x.
 * </li>
 * <li><h3>time()</h3>
 *     .
 * </li>
 * <li><h3>unplot(i)</h3>
 *     i  ,  NaN.
 * </li>
 * <li><h3>show()</h3>
 *    .
 * </li>
 * </ul>
 * 
 * @author Sergey Siryk
 */
final public class Functions {

	private Functions() {		
	}	

	static final PrintStream out = System.out;			
	
	static public Function getFunction(String name) {
		Function f = functions.get(name);
		if(f==null) {
			throw new Error("Cannot find function:"+name);
		}
		return f;
	}
	
	static abstract private class FunctionAction implements Function {
		public FunctionAction(String functionName, Integer ... numValidParms) {
			name=functionName;
			for(Integer vp:numValidParms) {
				validParms.add(vp);
			}
		}

		final List<Integer> validParms = new ArrayList<Integer>(); 
		
		void checkParametersNumber(int actual) {
			if(validParms.size()==0)
				return;
			for(Integer vp:validParms) {
				if(vp==actual)
					return;
			}
			throw new Error("Invalid number of parameters in "+toString()+" :"+actual+" parameter(s).");
		}

		public String toString() {
			return name+"()";
		}

		public String getName() {
			return name;
		}
		
		final String name;
		
		final public double evaluate(List<Double> params) {
			checkParametersNumber(params.size());
			return evaluateParms(params);
		}
		
		abstract double evaluateParms(List<Double> params);
	}

	abstract static private class FunctionActionNoParms extends FunctionAction {
		public FunctionActionNoParms(String functionName) {
			super(functionName, 0);
		}

		public double evaluateParms(List<Double> params) {			
			return evaluate();
		}

		abstract public double evaluate();			
	}

	abstract static private class FunctionAction1Parm extends FunctionAction {
		public FunctionAction1Parm(String functionName) {
			super(functionName, 1);
		}

		public double evaluateParms(List<Double> params) {			
			return evaluate(params.get(0));
		}

		abstract public double evaluate(double parm1);			
	}	
	
	static private Map<String,Function> functions = 
		new HashMap<String,Function>(); 	
		
	static void addFunction(Function fa) {
		functions.put(fa.getName(), fa);
	}

    static final BufferedReader reader = new BufferedReader(
            new InputStreamReader(System.in));	

	static {
		addFunction(new FunctionAction1Parm("abs") {
			public double evaluate(double parm1) {
				return Math.abs(parm1);
			}
		});
		addFunction(new FunctionAction("random",0,1) {
			double evaluateParms(List<Double> params) {
				if(params.size()==0)
					return Math.random();
				return params.get(0)*Math.random();
			}
		});
		addFunction(new FunctionAction1Parm("sin") {
			public double evaluate(double parm1) {
				return Math.sin(parm1);
			}
		});
		addFunction(new FunctionAction1Parm("cos") {
			public double evaluate(double parm1) {
				return Math.cos(parm1);
			}
		});
		addFunction(new FunctionAction1Parm("sqrt") {
			public double evaluate(double parm1) {
				return Math.sqrt(parm1);
			}
		});
		addFunction(new FunctionActionNoParms("pi") {
			public double evaluate() {
				return Math.PI;
			}
		});
		addFunction(new FunctionActionNoParms("e") {
			public double evaluate() {
				return Math.E;
			}
		});
		addFunction(new FunctionActionNoParms("NaN") {
			public double evaluate() {
				return Double.NaN;
			}
		});
		addFunction(new FunctionAction1Parm("isNaN") {
			public double evaluate(double parm1) {
				return DoubleOperations.check(Double.isNaN(parm1));
			}
		});
		addFunction(new FunctionActionNoParms("time") {
			public double evaluate() {
				return (new Date()).getTime();
			}
		});
		addFunction(new FunctionActionNoParms("readnum") {
			public double evaluate() {
				try {
					String s=reader.readLine();
					return new Double(s);
				} catch(Throwable t) {
					return Double.NaN;
				}
			}
		});					
				
		addFunction(new FunctionAction("print") {
			double evaluateParms(List<Double> params) {
				for(Double h:params) {
					out.print(h+" ");
				}
				return params.size(); 
			}
		});
		addFunction(new FunctionAction("println") {
			double evaluateParms(List<Double> params) {
				for(Double h:params) {
					out.print(h+" ");
				}
				out.println();
				return params.size(); 
			}
		});

		addFunction(new FunctionAction1Parm("delay") {
			public double evaluate(double parm1) {
				try {
					Thread.sleep((long)parm1);
				} catch (InterruptedException e) {
				}
				return 0; 
			}
		});
		
		// Graphics
		addFunction(new FunctionAction("plot",4) {
			double evaluateParms(List<Double> params) {
				double x=params.get(0);
				double y=params.get(1);
				int c=params.get(2).intValue();
				int ind=params.get(3).intValue();
				ContestWindow.plot(x, y, c, ind);
				return 0; 
			}
		});

		addFunction(new FunctionAction1Parm("unplot") {
			public double evaluate(double parm1) {
				ContestWindow.unplot((int)parm1);
				return 0; 
			}
		});

		addFunction(new FunctionActionNoParms("show") {
			public double evaluate() {
				ContestWindow.showWindow();
				return 0; 
			}
		});

		addFunction(new FunctionActionNoParms("repaint") {
			public double evaluate() {
				ContestWindow.repaintWindow();
				return 0; 
			}
		});		
		
		addFunction(new FunctionActionNoParms("hide") {
			public double evaluate() {
				ContestWindow.hideWindow();
				return 0; 
			}
		});		
		
		addFunction(new FunctionActionNoParms("keep") {
			public double evaluate() {
				ContestWindow.keep();
				return 0; 
			}
		});				
	}
}
