package com.ssv.test.airbase.contest;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.ssv.test.airbase.contest.Execution.Goto;

/**
 *        
 *   . ,      ,
 *    -  .
 * 
 * @author Sergey Siryk
 */

public final class Scope {	
	
	public static Variable getVariable(String name) {
		if(name.length()==0) {
			return NaN;
		}
		char c=name.charAt(0);
		if(! Character.isJavaIdentifierStart(c))
			return NaN;
		Variable v = _variables.get(name);
		if(v==null) {
			v=new DefaultVariable(name);
			_variables.put(name, v);
		}
		return v;
	}
	
	private static final Map<String, Variable> _variables =  
		new HashMap<String, Variable>(); 
	
	public static interface Variable {
		double getValue();
		void setValue(double v);
	}
	
	private static class DefaultVariable implements Variable {
		DefaultVariable(String name) {
			this.name=name;
		}
		
		final String name;
		double value=Double.NaN;

		public double getValue() {
			return value;
		}

		public void setValue(double v) {
			value=v;
		}
	}

	public static Variable makeConstantVariable(double d) {
		return new ConstantVariable(d);
	}
	
	private static class ConstantVariable implements Variable {
		ConstantVariable(double d) {
			value=d;
		}
		
		final double value;

		public double getValue() {
			return value;
		}

		public void setValue(double v) {
			throw new Error("Cannot assign to the constant:"+value);
		}
	}	
	
	final static Variable NaN = new NaNVariable();
	
	private static class NaNVariable implements Variable {
		NaNVariable() {
		}
		
		public double getValue() {
			return Double.NaN;
		}

		public void setValue(double v) {
		}
	}	

	public static class Mark {
		public Mark(int pos) {
			this.pos=pos;
			forwards = null;
		}
		
		public int getPos() {
			return pos;
		}

		public void setPos(int p) {
			pos=p;
		}		
		
		public Mark() {
			forwards = new ArrayList<Execution.Goto>();
		}
		
		public void setMarkers(int pos) {
			for(Execution.Goto gt:forwards) {
				gt.setMarker(pos);
			}			
		}

		public void addForward(Goto gt) {
			forwards.add(gt);
		}
		
		final List<Goto> forwards;
		private int pos = -1; // -1 means not defined yet.
	}
		
	private static Map<String,Mark> marks = new HashMap<String,Mark>();
	
	public static Mark getMark(String markname) {
		return marks.get(markname);		
	}
	
	public static void putMark(String name, int steps) {
		marks.put(name, new Mark(steps));
	}
	
	public static Mark createMark(String name) {
		Mark mark = new Mark();		
		marks.put(name, mark);
		return mark;
	}	
	
	private Scope() {		
	}
}
