package com.ssv.test.airbase.contest.expressions;

import java.util.List;

import com.ssv.test.airbase.contest.Expression;
import com.ssv.test.airbase.contest.Scope;
import com.ssv.utils.parser.ParserException;
import com.ssv.utils.parser.Parser.ParserContext;

/**
 * This expression represents indexed variable, like a[x]
 */
public class ExpressionIndexed extends AssignableVariable {
	private static final long serialVersionUID = -2713411284562778314L;

	public ExpressionIndexed(List<Part> parts, ParserContext ctx) {
		super(ctx);
		this.parts = parts;
	}

	final List<Part> parts;
	
	public static interface Part {
		String evaluate() throws ParserException;
	}
	
	static public class StringPart implements Part {
		public StringPart(String str) {
			this.str=str;
		}
		
		final String str;

		public String evaluate() {
			return str;
		}

		public String toString() {
			return str;
		}
	}

	static public class NumPart implements Part {
		public NumPart(double d) {
			this.d=d;
		}
		
		public double getNumber() {
			return d;
		}
		
		final double d;

		public String evaluate() {
			return ""+d;
		}

		public String toString() {
			return ""+d;
		}
	}	
	
	static public class ExpressionPart implements Part {
		public ExpressionPart(Expression expr) {
			this.expr=expr;
		}
		
		final Expression expr;

		public String evaluate() throws ParserException {
			return ""+expr.evaluate();
		}

		public String toString() {
			return "["+expr.toString()+"]";
		}
	}	
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		for(Part part:parts) {
			sb.append(part.toString());
		}
		return sb.toString();
	}	
	
	public Scope.Variable getVariable() throws ParserException  {
		StringBuffer sname = new StringBuffer();
		for(Part part:parts) {
			sname.append(part.evaluate());
		}
		return Scope.getVariable(sname.toString());
	}
}
