package com.ssv.test.airbase.contest.operators;

import java.util.List;

import com.ssv.test.airbase.contest.Execution;
import com.ssv.test.airbase.contest.Operator;
import com.ssv.test.airbase.contest.Scope;
import com.ssv.utils.parser.ParserException;


/**
 *     .<p>
 * <pre><code>
 * 		{
 * 			operator1();
 * 			operator2();
 * 		}
 * </code></pre>
 * 
 * @author Sergey Siryk
 *
 */
public class BlockOperator implements Operator {

	public BlockOperator(List<Operator> operators) {
		this.operators = operators;
	}
	
	public void run() throws ParserException {
		/*
		 *       . 
		 */
		for(Operator op:operators) {
			op.run();
		}
	}

	final private List<Operator> operators;

	public void generate(Execution execution) {
		int pos = 0;
		while(pos<operators.size()) {
			Operator op = operators.get(pos++);
			while(op instanceof Marker) { // Next operator is marked.
				Marker marker = (Marker) op;
				Scope.Mark mark = Scope.getMark(marker.getName());
				if(mark!=null) { // There is links to this mark.
					mark.setPos(execution.size());
					mark.setMarkers(execution.size());
				} else
					Scope.putMark(marker.getName(), execution.size());
				op = operators.get(pos++);
			}
			op.generate(execution);
		}
	}
}
