package com.ssv.test.airbase.contest.operators;

import com.ssv.test.airbase.contest.DoubleOperations;
import com.ssv.test.airbase.contest.Execution;
import com.ssv.test.airbase.contest.Expression;
import com.ssv.test.airbase.contest.Operator;
import com.ssv.test.airbase.contest.Execution.ConditionalGoto;
import com.ssv.test.airbase.contest.Execution.Goto;
import com.ssv.utils.parser.ParserException;

/**
 * ,     :<p>
 * 
 * <pre><code>
 * 		if(condition) {
 * 			operator1;
 * 		} else {
 * 			operator2;
 * 	    }
 * </code></pre>
 * 
 *    :

 * <pre><code>
 * 		if(condition) {
 * 			operator1;
 * 		} 
 * </code></pre>
 * 
 * @author Sergey Siryk
 */

public class IfOperator implements Operator {

	/**
	 *      if.
	 *  
	 * @param condition -  
	 * @param operator - , ,   .
	 */
	public IfOperator(Expression condition, Operator operator) {
		this(condition,operator,null);
	}

	/**
	 *      if.
	 *  
	 * @param condition -  
	 * @param operator - , ,   .
	 * @param opElse - , ,    .
	 */
	public IfOperator(Expression condition, 
				Operator operator,
				Operator opElse) { 
		this.condition=condition;
		this.operator=operator;
		this.opElse=opElse;
	}	
	
	final private Expression condition;
	final protected Operator operator;
	final private Operator opElse;
	
	public Expression getCondition() {
		return condition;
	}

	public Operator getOperator() {
		return operator;
	}	

	public Operator getElse() {
		return opElse;
	}		
	
	/**
	 * @return true     , false otherwise.
	 * @throws ParserException 
	 */
	boolean ok() throws ParserException {
		return DoubleOperations.check(condition.evaluate());
	}
	
	public void run() throws ParserException {
		if(ok())
			operator.run();
	}

	int specialGoto = 0;
	
	public void generate(Execution execution) {
		ConditionalGoto cg = 
			new ConditionalGoto(getCondition());
		execution.add(cg);
		getOperator().generate(execution);
		if(getElse()!=null) {
			Goto gt = new Goto("*"+specialGoto++);
			execution.add(gt);
			cg.setMarker(execution.size());
			getElse().generate(execution);
			gt.setMarker(execution.size());
		} else
			cg.setMarker(execution.size());
	}
}
