package com.ssv.test.airbase.contest.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.HashMap;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.JComponent;
import javax.swing.JFrame;

public class ContestWindow extends JFrame {

	private static final long serialVersionUID = -3766456128002212796L;
	
	static private Dimension windowSize = new Dimension(800,600);
	
	private static class Plots extends JComponent {
		private static final long serialVersionUID = 7516551346974888780L;		
		private static final int point_size = 2; 

		Timer timer = new Timer();
		
		public Plots() {
			super();
			timer.schedule(new TimerTask() {
				public void run() {
					repaint();
				}
			}, 100, 500);
		}
		
		public void stop() {
			timer.cancel();
		}

		static final double INSET = 10;
		
		public void paint(Graphics g) {
			if(points.size()==0)
				return;
			
			int w=getWidth();
			int h=getHeight();
			
			if(points.size()==1 || (maxx==minx && maxy==miny)) {
				for(PointInfo pi:points.values()) {
					g.setColor(pi.c);
					g.fillRect(w/2, h/2, point_size, point_size);
				}
				return;
			}
			
			if(minx>maxx)
				return;
			if(miny>maxy)
				return;
			
			double factW = maxx-minx;
			double factH = maxy-miny;

			double screenW = getWidth() - 2*INSET;
			double screenH = getHeight() - 2*INSET;
			
			double ax = screenW/factW;			
			double ay = screenH/factH;
			
			int shiftx = 0;
			int shifty = 0;
			
			if(ax>ay) {
				ax=ay;
				shiftx=(int)(getWidth()-ax*(factW))/2;
			} else {
				ay=ax;
				shifty=(int)(getHeight()-ay*(factH))/2;
			}
				
			double bx = INSET - ax*minx;
			double by = INSET - ay*miny;
			
			g.setColor(Color.white);
			g.fillRect(0,0,getWidth(),getHeight());
			synchronized(points) {
				for(PointInfo pi:points.values()) {
					g.setColor(pi.c);
					int x = (int)(pi.x*ax+bx);
					int y = (int)(pi.y*ay+by);
					g.fillRect(x+shiftx, y+shifty, point_size, point_size);
				}
			}
		}
		
		final Map<Integer,PointInfo> points = new HashMap<Integer,PointInfo>();
		
		static private class PointInfo {
			
			public PointInfo(double x, double y, Color c)
			{
				this.x=x;
				this.y=y;
				this.c=c;
			}
			
			double x;
			double y;
			final Color c;
		}

		private double minx=Double.POSITIVE_INFINITY;
		private double maxx=Double.NEGATIVE_INFINITY;
		private double miny=Double.POSITIVE_INFINITY;
		private double maxy=Double.NEGATIVE_INFINITY;				
		
		private void checkSize() {
			minx=Double.POSITIVE_INFINITY;
			maxx=Double.NEGATIVE_INFINITY;
			miny=Double.POSITIVE_INFINITY;
			maxy=Double.NEGATIVE_INFINITY;				
			
			for(PointInfo p:points.values()) {
				checkPoint(p.x,p.y);
			}
		}

		private void checkPoint(double x, double y) {
			if(x<minx)
				minx=x;
			if(x>maxx)
				maxx=x;
			
			if(y<miny)
				miny=y;
			if(y>maxy)
				maxy=y;
		}
		
		public void plot(double x, double y, Color c, int index) {
			synchronized(points) {
				PointInfo pf = points.get(index);
				if(pf!=null) {
					unplot(index);
				}
				points.put(index, new PointInfo(x,y,c));
				checkPoint(x,y);
			}
		}

		public void unplot(int index) {
			synchronized(points) {
				points.remove(index);
				checkSize();
			}
		}
	}
	
	public ContestWindow() {
		Dimension screenSize = getToolkit().getScreenSize();
		setBounds((screenSize.width-windowSize.width)/2,
				(screenSize.height-windowSize.height)/2, 
				windowSize.width,windowSize.height);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0); // An Exit Listener
			}
		});
		getRootPane().setLayout(new BorderLayout());
		getRootPane().add(plots,BorderLayout.CENTER);
	}

	Plots plots = new Plots();
	
	static ContestWindow window = null;

	private void _plot(double x, double y, Color c, int index) {
			plots.plot(x, y, c, index);
	}

	private void _unplot(int index) {
			plots.unplot(index);
	}
	
	static private void checkWindow() {
		if(window==null) {
			window=new ContestWindow();
			window.setVisible(true);
		}
	}
	
	static public void plot(double x,double y, long color, int index) {		
		checkWindow();
		//color:  100610
		int r = ((int) (color/10000))%11;
		int g = ((int) (color/100))%11;
		int b = ((int) (color))%11;
		Color c = new Color( 25*r, 25*g, 25*b);
		window._plot(x, y, c, index);
	}

	static public void unplot(int index) {
		checkWindow();
		window._unplot(index);
	}
	
	static public void hideWindow() {
		checkWindow();
		window.setVisible(false);
	}

	static public void showWindow() {
		checkWindow();
		window.setVisible(true);
	}	

	static public void repaintWindow() {
		checkWindow();
		window.plots.repaint();
	}		
	
	public static void keep() {
		keepIt=true;
	}
	
	static boolean keepIt=false;
	
	static public void shutdownWindow() {
		if(window!=null && ! keepIt) {
			window.plots.stop();
			window.setVisible(false);
			window.dispose();
		}
	}		
	
	public static void main(String[] args) {
		if(true) {
			int i=0;
			while(i<1000) {
				double r=Math.random()*200;
				if(Math.random()>0.5) 
					r=Math.random()*150;
				if(Math.random()>0.5) 
					r=Math.random()*100;
				if(Math.random()>0.5) 
					r=Math.random()*50;
				double a=Math.random()*2*Math.PI;				
				double x=Math.sin(a)*r;
				double y=Math.cos(a)*r;
				long c=(long)(Math.random()*10*10000+Math.random()*10*100+Math.random()*10);
				plot(x+400,y+250,c,i+400);
				i=i+1;
			};
		} else {
			for(int i=0;i<10;i++)
				plot(i*10,-i*10,0,i);
		}
	}
}
