package com.ssv.utils;

import java.io.IOException;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;

import com.ssv.text.ListOperations;
import com.ssv.utils.test.SerializationTest;

final public class BuiltinMap<K,T extends BuiltinMap.Value<K>> extends 
HashMap<K,T> implements Iterable<T> {

	private static final long serialVersionUID = -5576741962574873834L;

	public BuiltinMap() {
		super();
	}
	
	final public void put(T item) {
		super.put(item.getKey(), item);
	}

	final public void add(T item) {
		put(item);
	}	
	
	final public T remove(T item) {
		return super.remove(item.getKey());
	}	
	
	public final boolean contains(T item) {
		return super.containsKey(item.getKey());
	}

	public final Iterator<T> iterator() {
		return values().iterator();
	}
	
	final public T put(K key,T item) {
		throw new Error("Method put(Key, Item) is not supported in BuiltinMap.");
	}		
	
	private void writeObject(java.io.ObjectOutputStream out) throws IOException 
	{
		out.writeInt(size());
		for(Iterator<T> i=values().iterator();i.hasNext();) {
			out.writeObject(i.next());
		}
	}
	
	
	@SuppressWarnings("unchecked")
	private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException
	{
		int sz=in.readInt();
		for(int i=0;i<sz;i++) {
			add((T)in.readObject());
		}
	}
		
	
	/**
	 * The class represent an entity where key stores in the same structure where
	 * entity is stored. An example:<br>
	 * <code>
	 * 		class Person {<br>
	 * 			String getSSN();<br>
	 * 			String getName();<br>
	 * 		}<br>
	 * </code><br>
	 * 
	 * getSSN() is a key to look for the record, and also it is one of a person's 
	 * attributes. 
	 * @author SSiryk
	 * @see com.ssv.utils.BuiltinMap
	 *
	 */
	public interface Value<T> extends Serializable {
		T getKey();
	}

		
	/* UNIT TESTING */
	
	private static class Book implements BuiltinMap.Value<String> {
		
		private static final long serialVersionUID = -5459584098301732808L;

		public Book(String ISDN, String author, String title) {
			this.ISDN=ISDN;
			this.author=author;
			this.title=title;
		}
		
		public String getKey() {
			return ISDN;
		}

		public String toString() {
			return ISDN+" "+author+":"+title;
		}
		
		private String ISDN, author, title;
	}

	@SuppressWarnings("unchecked")
	static public void main(String args[]) {
		BuiltinMap<String,Book> emap=new BuiltinMap<String,Book>();
		
		emap.add(new Book("M198001","Sam Clamsey","It was"));
		emap.add(new Book("M198002","Anna Smith","Modern psichology"));
		emap.add(new Book("M198003","Julius Verne","Captain Grant's Children"));
		emap.add(new Book("M198004","Leo Tolstoy","The War and the Peace"));

		System.out.println(ListOperations.toString(emap));
		
		SerializationTest.writeObject(emap, BuiltinMap.class.getName());
		
		System.out.println(emap.get("M198004"));
		
		BuiltinMap bm=(BuiltinMap)SerializationTest.readObject(BuiltinMap.class.getName());
		System.out.println(bm.get("M198003"));
	}
}
