package com.ssv.utils;

/**
 * This interface represents name in dotted format (like Java classes).
 * Parts of name could be represented by String or any other object. 
 * Example: root.branch1.leaf22
 * @author Sergey Siryk
 */

public interface CompoundName<Item> {
	/**
	 * @param divider
	 * @return string representation of the name.
	 */ 
	public String format(String divider, boolean showRoot);

	/**
	 * @return last part of the compound name. If name is <code>town.street.home.apt</code>,
	 * the getLastToken() returns <code>apt</code>.
	 */
	public Item getLastToken();
	
	/**
	 * Number of name's part.
	 */
	public int numTokens();
	
	/**
	 * @param index - index of the token we want to retrieve 
	 * (0- first one, numTokens()-1 - last one.)
	 * @return
	 */
	public Item getToken(int index);
	
	/**
	 * @return "Package" part of the name. For example, if name is
	 * <code>town.street.home.apt</code>, prefix will be
	 * <code>town.street.home</code>
	 */
	public CompoundName<Item> getPrefix();
	
	/**
	 * @return array of tokens.
	 */
	public Item [] toArray();
	
	static public String DOT = ".";
	static public String SLASH = "/";
}
