package com.ssv.utils.tree;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.ssv.utils.test.SerializationTest;

public class DefaultTree<Leaf extends DefaultTree<Leaf>> extends
		Tree<Leaf> {
	
	private static final long serialVersionUID = 7747247941440170907L;


	public DefaultTree() {
		super();
	}

	final protected boolean addNode(Leaf leaf) {
		return getList().add(leaf);
	}	

	final protected boolean removeNode(Leaf leaf) {
		return getList().remove(leaf);
	}

	/**
	 * @return number of subnodes of current node.
	 */
	final public int size() {
		return getList().size();
	}

	/**
	 * @param index
	 * @return Subnode in the specified position.
	 */
	final public Leaf getNode(int index) {
		return getList().get(index);
	}
		
	/**
	 * Searches for the first occurence of the given argument, testing for
	 * equality using the equals method.
	 */
	final int indexOf(Leaf subnode) {
		return getList().indexOf(subnode);
	}

	final public Iterator<Leaf> iterator() {
		return getList().iterator();
	}

	final private List<Leaf> getList() {
		if(list==null)
			list=new ArrayList<Leaf>();
		return list;
	}
	
	private List<Leaf> list;
	
	/*
	 * UNIT TEST
	 */
	
	@SuppressWarnings("unchecked")
	public static void main(String args[]) {
		class DT extends DefaultTree<DT> {

			private static final long serialVersionUID = 4472161784069473034L;

			public DT(String name) {
				super();
				this.name = name;
			}

			public String toString() {
				return name;
			}
									
			String name;
		}

		DT root = new DT("root");
		DT lev1 = new DT("1");
		DT lev2 = new DT("2");
		DT lev3 = new DT("3");
		root.add(lev1);
		root.add(lev2);
		root.add(lev3);
		lev2.add(new DT("1"));
		lev2.add(new DT("2"));
		lev2.add(new DT("3"));
		lev3.add(new DT("1"));

		SimplePrintOperation<DT> smo=new SimplePrintOperation<DT>(System.out) {
			void printItem(Tree item) {
				System.out.println(item.format());
			}			
		};
		
		root.prefix(smo);
		System.out.println("***");
		root.postfix(smo);
		System.out.println("***");
		System.out.println(lev3.format(".", true));
		
		SerializationTest.writeObject(root, DefaultTree.class.getName());
		Tree t = (Tree) SerializationTest.readObject(DefaultTree.class.getName());
		t.prefix(smo);		
	}
}
