package com.ssv.utils.tree;

/**
 * This is basic class to implement tree's action. Tree's route methods
 * (prefix, postfix etc) uses this action to perform certain operations on
 * the nodes in the tree.
 */
public interface ITreeAction<Leaf extends Tree<Leaf>> {
	/**
	 * Helper class to go through the hierarchy.
	 * 
	 * @param item -
	 *            item to perform operation on.
	 * @return null if operation should go on, non-null value if operation
	 *         should be aborted (for example, if we already found
	 *         something)
	 */
	public Leaf performAction(Tree<Leaf> tree);

	/**
	 * This method will be invoked when route procedure enters to new level.
	 */
	public void enter();

	/**
	 * This method will be invoked when route procedure exits to previous
	 * level.
	 */
	public void exit();
}

